@@ -1,4 +1,11 @@
-"""Import records from supplied EAD XML files."""
+"""Import records from supplied EAD XML files.
+
+Validates imported files against the EAD3 XSD, and also against the
+Django models' validators, partly due to small differences between the
+two (eg, an empty physfacet is valid in the XML, not in the Django
+model) and partly as a sanity check.
+
+"""
 
 import logging
 import os.path
@@ -176,26 +183,31 @@ class Command(BaseCommand):
     def _import_abstract(self, abstract, ead_obj):
         abstract_obj = Abstract(did=ead_obj, abstract=serialise(abstract))
         import_attrs(abstract, ABSTRACT_ATTRS, abstract_obj)
+        abstract_obj.full_clean()
         abstract_obj.save()
 
     def _import_accessrestrict(self, accessrestrict, ead_obj):
         accessrestrict_obj = AccessRestrict(
             archdesc=ead_obj, accessrestrict=serialise(accessrestrict))
         import_attrs(accessrestrict, ACCESSRESTRICT_ATTRS, accessrestrict_obj)
+        accessrestrict_obj.full_clean()
         accessrestrict_obj.save()
 
     def _import_accruals(self, accruals, ead_obj):
         accruals_obj = Accruals(archdesc=ead_obj, accruals=serialise(accruals))
         import_attrs(accruals, ACCRUALS_ATTRS, accruals_obj)
+        accruals_obj.full_clean()
         accruals_obj.save()
 
     def _import_acqinfo(self, acqinfo, ead_obj):
         acqinfo_obj = AcqInfo(archdesc=ead_obj, acqinfo=serialise(acqinfo))
         import_attrs(acqinfo, ACQINFO_ATTRS, acqinfo_obj)
+        acqinfo_obj.full_clean()
         acqinfo_obj.save()
 
     def _import_address(self, address, repository_obj):
         import_attrs(address, ADDRESS_ATTRS, repository_obj, 'address_')
+        repository_obj.full_clean()
         repository_obj.save()
         for addressline in xpath(address, 'e:addressline'):
             self._import_addressline(addressline, repository_obj)
@@ -204,6 +216,7 @@ class Command(BaseCommand):
         addressline_obj = RepositoryAddressLine(
             address=repository_obj, addressline=serialise(addressline))
         import_attrs(addressline, ADDRESSLINE_ATTRS, addressline_obj)
+        addressline_obj.full_clean()
         addressline_obj.save()
 
     def _import_agencycode(self, agencycode, ead_obj):
@@ -214,6 +227,7 @@ class Command(BaseCommand):
         agencyname_obj = AgencyName(
             maintenanceagency=ead_obj, agencyname=agencyname.text)
         import_attrs(agencyname, AGENCYNAME_ATTRS, agencyname_obj)
+        agencyname_obj.full_clean()
         agencyname_obj.save()
 
     def _import_agent(self, agent, maintenanceevent_obj):
@@ -231,12 +245,14 @@ class Command(BaseCommand):
         altformavail_obj = AltFormAvail(
             archdesc=ead_obj, altformavail=serialise(altformavail))
         import_attrs(altformavail, ALTFORMAVAIL_ATTRS, altformavail_obj)
+        altformavail_obj.full_clean()
         altformavail_obj.save()
 
     def _import_appraisal(self, appraisal, ead_obj):
         appraisal_obj = Appraisal(
             archdesc=ead_obj, appraisal=serialise(appraisal))
         import_attrs(appraisal, APPRAISAL_ATTRS, appraisal_obj)
+        appraisal_obj.full_clean()
         appraisal_obj.save()
 
     def _import_archdesc(self, archdesc, ead_obj):
@@ -296,22 +312,26 @@ class Command(BaseCommand):
         arrangement_obj = Arrangement(
             archdesc=ead_obj, arrangement=serialise(arrangement))
         import_attrs(arrangement, ARRANGEMENT_ATTRS, arrangement_obj)
+        arrangement_obj.full_clean()
         arrangement_obj.save()
 
     def _import_author(self, author, ead_obj):
         author_obj = Author(titlestmt=ead_obj, author=serialise(author))
         import_attrs(author, AUTHOR_ATTRS, author_obj)
+        author_obj.full_clean()
         author_obj.save()
 
     def _import_bibliography(self, bibliography, ead_obj):
         bibliography_obj = Bibliography(
             archdesc=ead_obj, bibliography=serialise(bibliography))
         import_attrs(bibliography, BIBLIOGRAPHY_ATTRS, bibliography_obj)
+        bibliography_obj.full_clean()
         bibliography_obj.save()
 
     def _import_bioghist(self, bioghist, ead_obj):
         bioghist_obj = BiogHist(archdesc=ead_obj, bioghist=serialise(bioghist))
         import_attrs(bioghist, BIOGHIST_ATTRS, bioghist_obj)
+        bioghist_obj.full_clean()
         bioghist_obj.save()
 
     def _import_citation(self, citation, parent_obj):
@@ -321,6 +341,7 @@ class Command(BaseCommand):
     def _import_container(self, container, ead_obj):
         container_obj = Container(did=ead_obj, container=serialise(container))
         import_attrs(container, CONTAINER_ATTRS, container_obj)
+        container_obj.full_clean()
         container_obj.save()
         return container_obj
 
@@ -364,12 +385,14 @@ class Command(BaseCommand):
         controlaccess_obj = ControlAccess(
             archdesc=ead_obj, controlaccess=serialise(controlaccess))
         import_attrs(controlaccess, CONTROLACCESS_ATTRS, controlaccess_obj)
+        controlaccess_obj.full_clean()
         controlaccess_obj.save()
 
     def _import_controlnote(self, controlnote, ead_obj):
         controlnote_obj = ControlNote(
             notestmt=ead_obj, controlnote=serialise(controlnote))
         import_attrs(controlnote, CONTROLNOTE_ATTRS, controlnote_obj)
+        controlnote_obj.full_clean()
         controlnote_obj.save()
 
     def _import_conventiondeclaration(self, conventiondeclaration, ead_obj):
@@ -385,12 +408,14 @@ class Command(BaseCommand):
                 conventiondeclaration, 'e:descriptivenote'):
             self._import_descriptivenote(
                 descriptivenote, conventiondeclaration_obj)
+        conventiondeclaration_obj.full_clean()
         conventiondeclaration_obj.save()
 
     def _import_custodhist(self, custodhist, ead_obj):
         custodhist_obj = CustodHist(
             archdesc=ead_obj, custodhist=serialise(custodhist))
         import_attrs(custodhist, CUSTODHIST_ATTRS, custodhist_obj)
+        custodhist_obj.full_clean()
         custodhist_obj.save()
 
     def _import_dao(self, dao, parent_obj):
@@ -402,6 +427,7 @@ class Command(BaseCommand):
         import_attrs(dao, DAO_ATTRS, dao_obj)
         for descriptivenote in xpath(dao, 'e:descriptivenote'):
             self._import_descriptivenote(descriptivenote, dao_obj)
+        dao_obj.full_clean()
         dao_obj.save()
 
     def _import_daoset(self, daoset, ead_obj):
@@ -409,6 +435,7 @@ class Command(BaseCommand):
         import_attrs(daoset, DAOSET_ATTRS, daoset_obj)
         for descriptivenote in xpath(daoset, 'e:descriptivenote'):
             self._import_descriptivenote(descriptivenote, daoset_obj)
+        daoset_obj.full_clean()
         daoset_obj.save()
         for dao in xpath(daoset, 'e:dao'):
             self._import_dao(dao, daoset_obj)
@@ -420,6 +447,7 @@ class Command(BaseCommand):
             self._import_fromtodate(fromdate, daterange_obj, 'fromdate')
         for todate in xpath(daterange, 'e:todate'):
             self._import_fromtodate(todate, daterange_obj, 'todate')
+        daterange_obj.full_clean()
         daterange_obj.save()
 
     def _import_datesingle(self, datesingle, parent_obj, datesingle_model):
@@ -427,6 +455,7 @@ class Command(BaseCommand):
         import_attrs(datesingle, DATESINGLE_ATTRS, datesingle_obj)
         if datesingle.text or len(datesingle):
             datesingle_obj.datesingle = serialise(datesingle)
+        datesingle_obj.full_clean()
         datesingle_obj.save()
 
     def _import_descriptivenote(self, descriptivenote, parent_obj, prefix=''):
@@ -489,12 +518,14 @@ class Command(BaseCommand):
     def _import_didnote(self, didnote, ead_obj):
         didnote_obj = DIdNote(did=ead_obj, didnote=serialise(didnote))
         import_attrs(didnote, DIDNOTE_ATTRS, didnote_obj)
+        didnote_obj.full_clean()
         didnote_obj.save()
 
     def _import_dimensions(self, dimensions, pds_obj, dimensions_model):
         dimensions_obj = dimensions_model(
             physdescstructured=pds_obj, dimensions=serialise(dimensions))
         import_attrs(dimensions, DIMENSIONS_ATTRS, dimensions_obj)
+        dimensions_obj.full_clean()
         dimensions_obj.save()
 
     def _import_ead(self, ead):
@@ -518,6 +549,7 @@ class Command(BaseCommand):
             import_attrs(ead, EAD_ATTRS, ead_obj)
             self._import_control(ead[0], ead_obj)
             self._import_archdesc(ead[1], ead_obj)
+            ead_obj.full_clean()
             ead_obj.save()
             reversion.set_comment('Created record from EAD3 XML.')
         return ead_obj
@@ -539,6 +571,7 @@ class Command(BaseCommand):
         import_attrs(eventdescription, EVENTDESCRIPTION_ATTRS,
                      eventdescription_obj)
         eventdescription_obj.eventdescription = eventdescription.text
+        eventdescription_obj.full_clean()
         eventdescription_obj.save()
 
     def _import_eventtype(self, eventtype, maintenanceevent_obj):
@@ -562,6 +595,7 @@ class Command(BaseCommand):
     def _import_fileplan(self, fileplan, ead_obj):
         fileplan_obj = FilePlan(archdesc=ead_obj, fileplan=serialise(fileplan))
         import_attrs(fileplan, FILEPLAN_ATTRS, fileplan_obj)
+        fileplan_obj.full_clean()
         fileplan_obj.save()
 
     def _import_fromtodate(self, fromtodate, daterange_obj, field_name):
@@ -584,6 +618,7 @@ class Command(BaseCommand):
     def _import_index(self, index, ead_obj):
         index_obj = Index(archdesc=ead_obj, index=serialise(index))
         import_attrs(index, INDEX_ATTRS, index_obj)
+        index_obj.full_clean()
         index_obj.save()
 
     def _import_langmaterial(self, langmaterial, ead_obj):
@@ -591,6 +626,7 @@ class Command(BaseCommand):
         import_attrs(langmaterial, LANGMATERIAL_ATTRS, langmaterial_obj)
         for descriptivenote in xpath(langmaterial, 'e:descriptivenote'):
             self._import_descriptivenote(descriptivenote, langmaterial_obj)
+        langmaterial_obj.full_clean()
         langmaterial_obj.save()
         for language in xpath(langmaterial, 'e:language'):
             self._import_language(
@@ -606,6 +642,7 @@ class Command(BaseCommand):
             langcode_obj = search_term('iso639-2', langcode)
             language_obj.langcode = langcode_obj
         language_obj.language = language.text
+        language_obj.full_clean()
         language_obj.save()
 
     def _import_languagedeclaration(self, ld, ead_obj):
@@ -630,6 +667,7 @@ class Command(BaseCommand):
             ld_obj.script_el_scriptcode = scriptcode
             for descriptivenote in xpath(ld, 'e:descriptivenote'):
                 self._import_descriptivenote(descriptivenote, ld_obj)
+            ld_obj.full_clean()
             ld_obj.save()
         ead_obj.languagedeclarations.add(ld_obj)
 
@@ -638,6 +676,7 @@ class Command(BaseCommand):
         import_attrs(languageset, LANGUAGESET_ATTRS, languageset_obj)
         for descriptivenote in xpath(languageset, 'e:descriptivenote'):
             self._import_descriptivenote(descriptivenote, languageset_obj)
+        languageset_obj.full_clean()
         languageset_obj.save()
         for language in xpath(languageset, 'e:language'):
             self._import_language(
@@ -649,6 +688,7 @@ class Command(BaseCommand):
         legalstatus_obj = LegalStatus(
             archdesc=ead_obj, legalstatus=serialise(legalstatus))
         import_attrs(legalstatus, LEGALSTATUS_ATTRS, legalstatus_obj)
+        legalstatus_obj.full_clean()
         legalstatus_obj.save()
 
     def _import_localcontrol(self, localcontrol, ead_obj):
@@ -668,6 +708,7 @@ class Command(BaseCommand):
                          'datesingle_')
             if datesingle.text or len(datesingle):
                 localcontrol_obj.datesingle = serialise(datesingle)
+        localcontrol_obj.full_clean()
         localcontrol_obj.save()
 
     def _import_localtypedeclaration(self, ltd, ead_obj):
@@ -679,6 +720,7 @@ class Command(BaseCommand):
             self._import_citation(citation, ltd_obj)
         for descriptivenote in xpath(ltd, 'e:descriptivenote'):
             self._import_descriptivenote(descriptivenote, ltd_obj)
+        ltd_obj.full_clean()
         ltd_obj.save()
 
     def _import_maintenanceagency(self, maintenanceagency, ead_obj):
@@ -705,6 +747,7 @@ class Command(BaseCommand):
         self._import_agenttype(agenttype, maintenanceevent_obj)
         agent = maintenanceevent[3]
         self._import_agent(agent, maintenanceevent_obj)
+        maintenanceevent_obj.full_clean()
         maintenanceevent_obj.save()
         for eventdescription in maintenanceevent[4:]:
             self._import_eventdescription(
@@ -726,6 +769,7 @@ class Command(BaseCommand):
         materialspec_obj = MaterialSpec(
             did=ead_obj, materialspec=serialise(materialspec))
         import_attrs(materialspec, MATERIALSPEC_ATTRS, materialspec_obj)
+        materialspec_obj.full_clean()
         materialspec_obj.save()
 
     def _import_name(self, name, name_model, part_model):
@@ -740,6 +784,7 @@ class Command(BaseCommand):
         except name_model.DoesNotExist:
             name_obj = name_model(assembled_name=assembled_name)
             import_attrs(name, NAME_ATTRS, name_obj)
+            name_obj.full_clean()
             name_obj.save()
             for idx, part in enumerate(name):
                 self._import_part(part, name_obj, part_model, idx)
@@ -759,17 +804,20 @@ class Command(BaseCommand):
     def _import_odd(self, odd, ead_obj):
         odd_obj = ODD(archdesc=ead_obj, odd=serialise(odd))
         import_attrs(odd, ODD_ATTRS, odd_obj)
+        odd_obj.full_clean()
         odd_obj.save()
 
     def _import_originalsloc(self, originalsloc, ead_obj):
         originalsloc_obj = OriginalsLoc(
             archdesc=ead_obj, originalsloc=serialise(originalsloc))
         import_attrs(originalsloc, ORIGINALSLOC_ATTRS, originalsloc_obj)
+        originalsloc_obj.full_clean()
         originalsloc_obj.save()
 
     def _import_origination(self, origination, ead_obj):
         origination_obj = Origination(did=ead_obj)
         import_attrs(origination, ORIGINATION_ATTRS, origination_obj)
+        origination_obj.full_clean()
         origination_obj.save()
         for corpname in xpath(origination, 'e:corpname'):
             name_obj = self._import_name(corpname, CorpName, CorpNamePart)
@@ -789,34 +837,40 @@ class Command(BaseCommand):
             maintenanceagency=ead_obj, otheragencycode=otheragencycode.text)
         import_attrs(otheragencycode, OTHERAGENCYCODE_ATTRS,
                      otheragencycode_obj)
+        otheragencycode_obj.full_clean()
         otheragencycode_obj.save()
 
     def _import_otherfindaid(self, otherfindaid, ead_obj):
         otherfindaid_obj = OtherFindAid(
             archdesc=ead_obj, otherfindaid=serialise(otherfindaid))
         import_attrs(otherfindaid, OTHERFINDAID_ATTRS, otherfindaid_obj)
+        otherfindaid_obj.full_clean()
         otherfindaid_obj.save()
 
     def _import_otherrecordid(self, otherrecordid, ead_obj):
         otherrecordid_obj = OtherRecordID(
             control=ead_obj, otherrecordid=otherrecordid.text)
         import_attrs(otherrecordid, OTHERRECORDID_ATTRS, otherrecordid_obj)
+        otherrecordid_obj.full_clean()
         otherrecordid_obj.save()
 
     def _import_part(self, part, name_obj, part_model, position):
         part_obj = part_model(
             name=name_obj, part=serialise(part), order=position+1)
         import_attrs(part, PART_ATTRS, part_obj)
+        part_obj.full_clean()
         part_obj.save()
 
     def _import_physdesc(self, physdesc, ead_obj):
         physdesc_obj = PhysDesc(did=ead_obj, physdesc=serialise(physdesc))
         import_attrs(physdesc, PHYSDESC_ATTRS, physdesc_obj)
+        physdesc_obj.full_clean()
         physdesc_obj.save()
 
     def _import_physdescset(self, physdescset, ead_obj):
         physdescset_obj = PhysDescSet(did=ead_obj)
         import_attrs(physdescset, PHYSDESCSET_ATTRS, physdescset_obj)
+        physdescset_obj.full_clean()
         physdescset_obj.save()
         for pds in xpath(physdescset, 'e:physdescstructured'):
             self._import_physdescstructured(pds, physdescset_obj)
@@ -838,6 +892,7 @@ class Command(BaseCommand):
             self._import_unittype(unittype, pds_obj)
         for descriptivenote in xpath(pds, 'e:descriptivenote'):
             self._import_descriptivenote(descriptivenote, pds_obj)
+        pds_obj.full_clean()
         pds_obj.save()
         for dimensions in xpath(pds, 'e:dimensions'):
             self._import_dimensions(dimensions, pds_obj, dimensions_model)
@@ -848,11 +903,13 @@ class Command(BaseCommand):
         physfacet_obj = physfacet_model(
             physdescstructured=pds_obj, physfacet=serialise(physfacet))
         import_attrs(physfacet, PHYSFACET_ATTRS, physfacet_obj)
+        physfacet_obj.full_clean()
         physfacet_obj.save()
 
     def _import_physloc(self, physloc, ead_obj):
         physloc_obj = PhysLoc(did=ead_obj, physloc=serialise(physloc))
         import_attrs(physloc, PHYSLOC_ATTRS, physloc_obj)
+        physloc_obj.full_clean()
         physloc_obj.save()
         return physloc_obj
 
@@ -864,18 +921,21 @@ class Command(BaseCommand):
     def _import_phystech(self, phystech, ead_obj):
         phystech_obj = PhysTech(archdesc=ead_obj, phystech=serialise(phystech))
         import_attrs(phystech, PHYSTECH_ATTRS, phystech_obj)
+        phystech_obj.full_clean()
         phystech_obj.save()
 
     def _import_prefercite(self, prefercite, ead_obj):
         prefercite_obj = PreferCite(
             archdesc=ead_obj, prefercite=serialise(prefercite))
         import_attrs(prefercite, PREFERCITE_ATTRS, prefercite_obj)
+        prefercite_obj.full_clean()
         prefercite_obj.save()
 
     def _import_processinfo(self, processinfo, ead_obj):
         processinfo_obj = ProcessInfo(
             archdesc=ead_obj, processinfo=serialise(processinfo))
         import_attrs(processinfo, PROCESSINFO_ATTRS, processinfo_obj)
+        processinfo_obj.full_clean()
         processinfo_obj.save()
 
     def _import_publicationstatus(self, publicationstatus, ead_obj):
@@ -902,6 +962,7 @@ class Command(BaseCommand):
         rm_obj = RelatedMaterial(
             archdesc=ead_obj, relatedmaterial=serialise(rm))
         import_attrs(rm, RELATEDMATERIAL_ATTRS, rm_obj)
+        rm_obj.full_clean()
         rm_obj.save()
 
     def _import_relation(self, relation, ead_obj):
@@ -923,6 +984,7 @@ class Command(BaseCommand):
             self._import_geogname(geogname, relation_obj)
         for descriptivenote in xpath(relation, 'e:descriptivenote'):
             self._import_descriptivenote(descriptivenote, relation_obj)
+        relation_obj.full_clean()
         relation_obj.save()
 
     def _import_relations(self, relations, ead_obj):
@@ -934,6 +996,7 @@ class Command(BaseCommand):
         relationentry_obj = RelationEntry(relation=relation_obj)
         import_attrs(relationentry, RELATIONENTRY_ATTRS, relationentry_obj)
         relationentry_obj.relationentry = relationentry.text
+        relationentry_obj.full_clean()
         relationentry_obj.save()
 
     def _import_repository(self, repository, ead_obj):
@@ -947,6 +1010,7 @@ class Command(BaseCommand):
             pass
         repository_obj = Repository()
         import_attrs(repository, REPOSITORY_ATTRS, repository_obj)
+        repository_obj.full_clean()
         repository_obj.save()
         ead_obj.repositories.add(repository_obj)
         for corpname in xpath(repository, 'e:corpname'):
@@ -970,6 +1034,7 @@ class Command(BaseCommand):
         import_attrs(representation, REPRESENTATION_ATTRS, representation_obj)
         if representation.text:
             representation_obj.representation = representation.text
+        representation_obj.full_clean()
         representation_obj.save()
 
     def _import_rightsdeclaration(self, rightsdeclaration, ead_obj):
@@ -981,12 +1046,14 @@ class Command(BaseCommand):
             self._import_citation(citation, rd_obj)
         for descriptivenote in xpath(rightsdeclaration, 'e:descriptivenote'):
             self._import_descriptivenote(descriptivenote, rd_obj)
+        rd_obj.full_clean()
         rd_obj.save()
 
     def _import_scopecontent(self, scopecontent, ead_obj):
         scopecontent_obj = ScopeContent(
             archdesc=ead_obj, scopecontent=serialise(scopecontent))
         import_attrs(scopecontent, SCOPECONTENT_ATTRS, scopecontent_obj)
+        scopecontent_obj.full_clean()
         scopecontent_obj.save()
 
     def _import_script(self, script, languageset_obj):
@@ -998,6 +1065,7 @@ class Command(BaseCommand):
         if scriptcode:
             script_obj.scriptcode = search_term('iso15924', scriptcode)
         script_obj.script_el = script.text or ''
+        script_obj.full_clean()
         script_obj.save()
 
     def _import_separatedmaterial(self, separatedmaterial, ead_obj):
@@ -1005,6 +1073,7 @@ class Command(BaseCommand):
             archdesc=ead_obj, separatedmaterial=serialise(separatedmaterial))
         import_attrs(
             separatedmaterial, SEPARATEDMATERIAL_ATTRS, separatedmaterial_obj)
+        separatedmaterial_obj.full_clean()
         separatedmaterial_obj.save()
 
     def _import_seriesstmt(self, seriesstmt, ead_obj):
@@ -1021,12 +1090,14 @@ class Command(BaseCommand):
             self._import_objectxmlwrap(objectxmlwrap, source_obj)
         for descriptivenote in xpath(source, 'e:descriptivenote'):
             self._import_descriptivenote(descriptivenote, source_obj)
+        source_obj.full_clean()
         source_obj.save()
 
     def _import_sourceentry(self, sourceentry, source_obj):
         sourceentry_obj = SourceEntry(
             source=source_obj, sourceentry=sourceentry.text)
         import_attrs(sourceentry, SOURCEENTRY_ATTRS, sourceentry_obj)
+        sourceentry_obj.full_clean()
         sourceentry_obj.save()
 
     def _import_sources(self, sources, ead_obj):
@@ -1037,12 +1108,14 @@ class Command(BaseCommand):
     def _import_sponsor(self, sponsor, ead_obj):
         sponsor_obj = Sponsor(titlestmt=ead_obj, sponsor=serialise(sponsor))
         import_attrs(sponsor, SPONSOR_ATTRS, sponsor_obj)
+        sponsor_obj.full_clean()
         sponsor_obj.save()
 
     def _import_subtitle(self, subtitle, ead_obj):
         subtitle_obj = Subtitle(
             titlestmt=ead_obj, subtitle=serialise(subtitle))
         import_attrs(subtitle, SUBTITLE_ATTRS, subtitle_obj)
+        subtitle_obj.full_clean()
         subtitle_obj.save()
 
     def _import_term(self, term, localcontrol_obj):
@@ -1055,6 +1128,7 @@ class Command(BaseCommand):
         titleproper_obj = TitleProper(
             titlestmt=ead_obj, titleproper=serialise(titleproper))
         import_attrs(titleproper, TITLEPROPER_ATTRS, titleproper_obj)
+        titleproper_obj.full_clean()
         titleproper_obj.save()
 
     def _import_titlestmt(self, titlestmt, ead_obj):
@@ -1071,6 +1145,7 @@ class Command(BaseCommand):
     def _import_unitdate(self, unitdate, ead_obj):
         unitdate_obj = UnitDate(did=ead_obj, unitdate=serialise(unitdate))
         import_attrs(unitdate, UNITDATE_ATTRS, unitdate_obj)
+        unitdate_obj.full_clean()
         unitdate_obj.save()
 
     def _import_unitdatestructured(self, uds, ead_obj):
@@ -1078,6 +1153,7 @@ class Command(BaseCommand):
         import_attrs(uds, UNITDATESTRUCTURED_ATTRS, uds_obj)
         for dateset in xpath(uds, 'e:dateset'):
             import_attrs(dateset, DATESET_ATTRS, uds_obj, 'dateset_')
+        uds_obj.full_clean()
         uds_obj.save()
         for daterange in xpath(uds, './/e:daterange'):
             self._import_daterange(
@@ -1089,11 +1165,13 @@ class Command(BaseCommand):
     def _import_unitid(self, unitid, ead_obj):
         unitid_obj = UnitId(did=ead_obj, unitid=serialise(unitid))
         import_attrs(unitid, UNITID_ATTRS, unitid_obj)
+        unitid_obj.full_clean()
         unitid_obj.save()
 
     def _import_unittitle(self, unittitle, ead_obj):
         unittitle_obj = UnitTitle(did=ead_obj, unittitle=serialise(unittitle))
         import_attrs(unittitle, UNITTITLE_ATTRS, unittitle_obj)
+        unittitle_obj.full_clean()
         unittitle_obj.save()
 
     def _import_unittype(self, unittype, physdescstructured_obj):
@@ -1105,4 +1183,5 @@ class Command(BaseCommand):
         userestrict_obj = UseRestrict(
             archdesc=ead_obj, userestrict=serialise(userestrict))
         import_attrs(userestrict, USERESTRICT_ATTRS, userestrict_obj)
+        userestrict_obj.full_clean()
         userestrict_obj.save()
