@@ -351,7 +351,6 @@ class AbbrMixin(models.Model):
 
 class CitationMixin(models.Model):
     """Corresponds to EAD3 citation."""
-
     citation = models.TextField()
     citation_actuate = deepcopy(actuate)
     citation_altrender = deepcopy(altrender)
@@ -440,7 +439,7 @@ class DateSingleMixin(CommonMixin, LangScriptMixin, LocalTypeMixin):
 
 class DescriptiveNoteMixin(models.Model):
     """Corresponds to EAD3 descriptivenote."""
-    descriptivenote = models.TextField()
+    descriptivenote = models.TextField(blank=True)
     descriptivenote_altrender = deepcopy(altrender)
     descriptivenote_audience = deepcopy(audience)
     descriptivenote_encodinganalog = deepcopy(encodinganalog)
@@ -496,7 +495,7 @@ class ObjectXMLWrapMixin(models.Model):
     objectxmlwrap_ead_id = deepcopy(ead_id)
     objectxmlwrap_lang = deepcopy(lang)
     objectxmlwrap_script = deepcopy(script)
-    objectxmlwrap = models.TextField()
+    objectxmlwrap = models.TextField(blank=True)
 
     class Meta:
         abstract = True
@@ -515,7 +514,7 @@ class PartMixin(AccessMixin, CommonMixin, EncodingAnalogMixin, LangScriptMixin,
     """
     part = models.TextField()
     order = models.PositiveSmallIntegerField()
-    plain_name = models.TextField()
+    plain_name = models.TextField(blank=True)
 
     def save(self, *args, **kwargs):
         part_xml = etree.fromstring('<wrapper>{}</wrapper>'.format(
@@ -546,7 +545,7 @@ class PhysDescStructuredMixin(CommonMixin, CoverageMixin, EncodingAnalogMixin,
     otherphysdescstructuredtype = models.CharField(
         blank=True, max_length=64, validators=[validate_token])
     quantity_altrender = deepcopy(altrender)
-    quantity_approximate = models.BooleanField(null=True)
+    quantity_approximate = models.BooleanField(blank=True, null=True)
     quantity_audience = deepcopy(audience)
     quantity_encodinganalog = deepcopy(encodinganalog)
     quantity_ead_id = deepcopy(ead_id)
@@ -1599,7 +1598,7 @@ class PhysDescSet(CommonMixin, CoverageMixin, EncodingAnalogMixin, LabelMixin,
                   LangScriptMixin, DescriptiveNoteMixin):
     """Corresponds to EAD3 physdescset."""
     did = models.ForeignKey(EAD, on_delete=models.CASCADE)
-    parallel = models.BooleanField(null=True)
+    parallel = models.BooleanField(blank=True, null=True)
 
 
 @reversion.register(follow=['dimensions_set', 'physfacet_set'])
@@ -1736,7 +1735,7 @@ class Repository(CommonMixin, EncodingAnalogMixin, LabelMixin, LangScriptMixin,
     address_ead_id = deepcopy(ead_id)
     address_lang = deepcopy(lang)
     address_script = deepcopy(script)
-    assembled_name = models.TextField()
+    assembled_name = models.TextField(blank=True)
     corpnames = models.ManyToManyField(CorpName)
     famnames = models.ManyToManyField(FamName)
     names = models.ManyToManyField(Name)
